==> llm-docs-0.1.5.tgz: package/docs/integration/openapi/index.md <==
---
id: integration-openapi
---
# API Guidelines

This document defines the API design standards and implementation requirements for the LLM Platform AI Platform, ensuring strict OpenAPI 3.1 compliance and resource-driven execution patterns.

## Core Principles

1. **OpenAPI 3.1 Compliance**: All APIs must be explicitly defined using OpenAPI 3.1
2. **Resource-Driven Design**: All operations follow resource-oriented patterns
3. **Full Platform Validation**: Continuous validation across all environments
4. **Enterprise Security**: JWT/OAuth2, RBAC, and automated scanning
5. **Observability**: Langfuse, Helicone, and OpenLLMetry integration

## OpenAPI Specification Requirements

All APIs must be defined using OpenAPI 3.1 and validated against our standards:

```bash
# Validate OpenAPI specification
llmcli openapi validate
llmcli openapi lint
llmcli openapi test
```

Required OpenAPI components:

```yaml
openapi: 3.1.0
info:
  title: LLM Platform AI Platform API
  version: '1.0.0'
  description: |
    Enterprise AI Platform API
    - OpenAPI 3.1 compliant
    - Resource-driven execution
    - Full platform validation
  contact:
    name: Platform Team
    email: platform@bluefly.io
  license:
    name: Proprietary
servers:
  - url: https://api.bluefly.io/v1
    description: Production API
  - url: https://staging-api.bluefly.io/v1
    description: Staging API
components:
  securitySchemes:
    bearerAuth:
      type: http
      scheme: bearer
      bearerFormat: JWT
    apiKeyAuth:
      type: apiKey
      in: header
      name: X-API-Key
  schemas:
    Error:
      type: object
      required:
        - code
        - message
      properties:
        code:
          type: string
          enum: [invalid_request, unauthorized, forbidden, not_found, conflict, rate_limited, internal_error]
        message:
          type: string
        details:
          type: array
          items:
            $ref: '#/components/schemas/ErrorDetail'
        requestId:
          type: string
          format: uuid
    ErrorDetail:
      type: object
      required:
        - field
        - issue
      properties:
        field:
          type: string
        issue:
          type: string
        message:
          type: string
  responses:
    Error:
      description: Error response
      content:
        application/json:
          schema:
            $ref: '#/components/schemas/Error'
    Unauthorized:
      description: Authentication required
      content:
        application/json:
          schema:
            $ref: '#/components/schemas/Error'
    Forbidden:
      description: Insufficient permissions
      content:
        application/json:
          schema:
            $ref: '#/components/schemas/Error'
    NotFound:
      description: Resource not found
      content:
        application/json:
          schema:
            $ref: '#/components/schemas/Error'
    RateLimited:
      description: Rate limit exceeded
      content:
        application/json:
          schema:
            $ref: '#/components/schemas/Error'
      headers:
        Retry-After:
          schema:
            type: integer
            format: int32
```

## Resource-Driven Execution

All API endpoints must follow resource-oriented patterns:

1. **Resource Identification**
   - Use nouns for resource names
   - Follow hierarchical relationships
   - Maintain consistent naming conventions

2. **Resource Operations**
   - GET: Retrieve resource(s)
   - POST: Create resource
   - PUT: Replace resource
   - PATCH: Update resource
   - DELETE: Remove resource

3. **Resource Relationships**
   - Use nested resources for relationships
   - Maintain referential integrity
   - Follow RESTful principles

Example resource structure:

```yaml
paths:
  /models:
    get:
      summary: List models
      operationId: listModels
      tags: [Models]
      security:
        - bearerAuth: []
      parameters:
        - $ref: '#/components/parameters/PageSize'
        - $ref: '#/components/parameters/PageToken'
        - $ref: '#/components/parameters/Filter'
        - $ref: '#/components/parameters/Sort'
      responses:
        '200':
          description: Successful response
          content:
            application/json:
              schema:
                $ref: '#/components/schemas/ModelList'
        '401':
          $ref: '#/components/responses/Unauthorized'
        '403':
          $ref: '#/components/responses/Forbidden'
        '429':
          $ref: '#/components/responses/RateLimited'
  /models/{modelId}:
    parameters:
      - name: modelId
        in: path
        required: true
        schema:
          type: string
          format: uuid
    get:
      summary: Get model
      operationId: getModel
      tags: [Models]
      security:
        - bearerAuth: []
      responses:
        '200':
          description: Successful response
          content:
            application/json:
              schema:
                $ref: '#/components/schemas/Model'
        '401':
          $ref: '#/components/responses/Unauthorized'
        '403':
          $ref: '#/components/responses/Forbidden'
        '404':
          $ref: '#/components/responses/NotFound'
        '429':
          $ref: '#/components/responses/RateLimited'
```

## Response Standards

### Success Response

```json
{
  "data": {
    "id": "model-123",
    "type": "text-generation",
    "name": "GPT-4",
    "parameters": {
      "temperature": 0.7,
      "maxTokens": 1024
    },
    "metadata": {
      "team": "research",
      "project": "content-generation"
    },
    "createdAt": "2024-03-15T12:00:00Z",
    "updatedAt": "2024-03-15T14:30:00Z"
  },
  "meta": {
    "requestId": "req-abc-123",
    "timestamp": "2024-03-15T14:35:12Z",
    "version": "1.0.0"
  }
}
```

### Collection Response

```json
{
  "data": [
    {
      "id": "model-123",
      "type": "text-generation",
      "name": "GPT-4"
    }
  ],
  "meta": {
    "pagination": {
      "total": 48,
      "count": 1,
      "page": 1,
      "pages": 48,
      "next": "/api/v1/models?page=2&pageSize=1"
    },
    "requestId": "req-abc-123",
    "timestamp": "2024-03-15T14:35:12Z",
    "version": "1.0.0"
  }
}
```

## Error Handling

All errors must follow the standard error format:

```json
{
  "error": {
    "code": "invalid_request",
    "message": "The request was invalid",
    "details": [
      {
        "field": "name",
        "issue": "required",
        "message": "Name is required"
      }
    ],
    "requestId": "req-abc-123"
  }
}
```

## Security Requirements

1. **Authentication**
   - JWT Bearer tokens for all requests
   - API keys for service-to-service communication
   - OAuth2 for user authentication

2. **Authorization**
   - Role-based access control (RBAC)
   - Resource-level permissions
   - Audit logging for all operations

3. **Rate Limiting**
   - Per-endpoint rate limits
   - Per-client rate limits
   - Burst allowance for authenticated clients

## Observability

All endpoints must include:

1. **Request Tracking**
   - Unique request ID
   - Client ID
   - User ID (if authenticated)
   - Timestamp

2. **Performance Metrics**
   - Response time
   - Resource usage
   - Error rates

3. **Logging**
   - Request/response logging
   - Error logging
   - Audit logging

## Versioning

1. **API Versioning**
   - URL path versioning (/v1, /v2)
   - Content negotiation for minor versions
   - Deprecation notices for major versions

2. **Schema Versioning**
   - OpenAPI version tracking
   - Schema evolution guidelines
   - Backward compatibility requirements

## Platform Validation

Every API must pass:

1. **OpenAPI Validation**
   - Schema validation
   - Security validation
   - Documentation completeness

2. **Performance Testing**
   - Response time benchmarks
   - Throughput testing
   - Load testing

3. **Security Scanning**
   - Vulnerability scanning
   - Dependency scanning
   - Compliance checking

Use `llmcli api validate` to run full API validation.

## URL Structure

Follow these URL structure conventions:

```
https://api.example.com/api/v1/resource
https://api.example.com/api/v1/resource/{id}
https://api.example.com/api/v1/resource/{id}/subresource
```

- Base path includes `/api/v\{version_number\}/`
- Use plural nouns for resources (e.g., `/models` not `/model`)
- Use kebab-case for multi-word resource names (e.g., `/model-variants`)
- Use query parameters for filtering, sorting, and pagination

## HTTP Methods

Use standard HTTP methods appropriately:

| Method | Usage | Example |
|--------|-------|---------|
| GET | Retrieve resources | GET /api/v1/models |
| POST | Create resources | POST /api/v1/models |
| PUT | Replace resources | PUT /api/v1/models/123 |
| PATCH | Partial update | PATCH /api/v1/models/123 |
| DELETE | Remove resources | DELETE /api/v1/models/123 |

## Request & Response Format

### Request Structure

```json
{
  "name": "My Custom Model",
  "type": "text-generation",
  "parameters": {
    "temperature": 0.7,
    "max_tokens": 1024
  },
  "metadata": {
    "team": "research",
    "project": "content-generation"
  }
}
```

- Use camelCase for property names
- Group related properties into nested objects
- Keep the structure flat when possible
- Use appropriate data types

### Response Structure

```json
{
  "data": {
    "id": "model-123",
    "name": "My Custom Model",
    "type": "text-generation",
    "parameters": {
      "temperature": 0.7,
      "max_tokens": 1024
    },
    "metadata": {
      "team": "research",
      "project": "content-generation"
    },
    "createdAt": "2025-05-01T12:00:00Z",
    "updatedAt": "2025-05-15T14:30:00Z"
  },
  "meta": {
    "requestId": "req-abc-123",
    "timestamp": "2025-05-15T14:35:12Z"
  }
}
```

For collections:

```json
{
  "data": [
    {
      "id": "model-123",
      "name": "GPT-4",
      "type": "text-generation"
    },
    {
      "id": "model-456",
      "name": "DALL-E 3",
      "type": "image-generation"
    }
  ],
  "meta": {
    "pagination": {
      "total": 48,
      "count": 2,
      "page": 1,
      "pages": 24,
      "prev": null,
      "next": "/api/v1/models?page=2"
    },
    "requestId": "req-abc-123",
    "timestamp": "2025-05-15T14:35:12Z"
  }
}
```

## Error Handling

Use consistent error response format:

```json
{
  "error": {
    "code": "invalid_request",
    "message": "The request was invalid",
    "details": [
      {
        "field": "name",
        "issue": "required",
        "message": "Name is required"
      }
    ],
    "requestId": "req-abc-123"
  }
}
```

HTTP status codes:

| Code | Meaning | Example Usage |
|------|---------|---------------|
| 200 | OK | Successful GET, PUT, PATCH |
| 201 | Created | Successful POST that creates a resource |
| 204 | No Content | Successful DELETE |
| 400 | Bad Request | Invalid request format or parameters |
| 401 | Unauthorized | Missing or invalid authentication |
| 403 | Forbidden | Valid authentication but insufficient permissions |
| 404 | Not Found | Resource doesn't exist |
| 409 | Conflict | Resource already exists or other conflict |
| 422 | Unprocessable Entity | Semantic errors in request |
| 429 | Too Many Requests | Rate limit exceeded |
| 500 | Internal Server Error | Server-side error |

## Authentication

Use JWT Bearer tokens for authentication:

```
Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...
```

Include token validation middleware in all protected routes:

```javascript
app.use('/api/v1', authenticate(), routes);
```

## Versioning

Include version in the URL path:

```
/api/v1/models
/api/v2/models
```

Version compatibility guidelines:

- Minor version changes must be backward compatible
- Major version changes may break compatibility
- Maintain previous major versions for at least 6 months
- Document all breaking changes in the changelog

## Rate Limiting

Implement rate limiting on all endpoints:

```
X-RateLimit-Limit: 100
X-RateLimit-Remaining: 95
X-RateLimit-Reset: 1622568000
```

When limit is exceeded, return 429 Too Many Requests with Retry-After header.

## Pagination

Use cursor-based pagination for large collections:

```
/api/v1/models?limit=10&after=model-123
```

Include pagination metadata in response:

```json
"meta": {
  "pagination": {
    "total": 48,
    "count": 10,
    "after": "model-123",
    "hasMore": true,
    "next": "/api/v1/models?limit=10&after=model-132"
  }
}
```

## Filtering & Sorting

Support filtering via query parameters:

```
/api/v1/models?type=text-generation&status=active
```

Support sorting via `sort` parameter:

```
/api/v1/models?sort=name       # Ascending
/api/v1/models?sort=-name      # Descending
/api/v1/models?sort=type,-name # Multiple fields
```

## CORS Configuration

Configure CORS to allow necessary domains:

```javascript
app.use(cors({
  origin: ['https://app.example.com', 'https://admin.example.com'],
  methods: ['GET', 'POST', 'PUT', 'PATCH', 'DELETE'],
  allowedHeaders: ['Content-Type', 'Authorization'],
  exposedHeaders: ['X-RateLimit-Limit', 'X-RateLimit-Remaining', 'X-RateLimit-Reset'],
  credentials: true,
  maxAge: 86400 // 24 hours
}));
```

## Caching

Use HTTP caching headers:

```
Cache-Control: public, max-age=60
ETag: "33a64df551425fcc55e4d42a148795d9f25f89d4"
Last-Modified: Wed, 15 May 2025 14:30:00 GMT
```

Implement conditional requests with If-None-Match and If-Modified-Since headers.

## API Documentation

Document all APIs using OpenAPI specification and provide interactive documentation:

```bash
# Generate API documentation
npm run generate-api-docs

# Serve API documentation
npm run serve-api-docs
```

Documentation should include:

- Endpoint descriptions
- Request/response schemas
- Authentication requirements
- Error codes and messages
- Example requests and responses
- Rate limiting information

## API Validation & Testing

All APIs must have:

1. Schema validation for requests and responses
2. Automated tests for all endpoints
3. Contract tests to ensure API compatibility
4. Performance tests to ensure response time requirements

Example validation:

```javascript
const schema = Joi.object({
  name: Joi.string().required(),
  type: Joi.string().valid('text-generation', 'image-generation').required(),
  parameters: Joi.object({
    temperature: Joi.number().min(0).max(2).default(1),
    max_tokens: Joi.number().integer().min(1).max(4096).default(1024)
  }).default({})
});

const { error, value } = schema.validate(req.body);
if (error) {
  return res.status(400).json({
    error: {
      code: 'invalid_request',
      message: 'The request was invalid',
      details: error.details.map(d => ({
        field: d.path.join('.'),
        issue: d.type,
        message: d.message
      })),
      requestId: req.id
    }
  });
}
```

## API Gateway Configuration

All APIs are exposed through the API Gateway (bfapi) which provides:

- Authentication and authorization
- Rate limiting and throttling
- Request validation
- Response caching
- Logging and monitoring
- API key management

## API Monitoring

All APIs are monitored for:

- Request volume
- Response time
- Error rates
- Authentication failures
- Rate limit hits

Metrics are exposed via Prometheus endpoints and visualized in Grafana dashboards.

## Changelog

Document all API changes in a changelog:

```markdown
# API Changelog

## v1.1.0 - 2025-05-15

### Added
- New `status` field to model responses
- Support for sorting models by creation date

### Changed
- Increased rate limits from 100 to 200 requests per minute

## v1.0.0 - 2025-04-01

- Initial release of the API
```

==> llm-docs-1.0.1.tgz: package/docs/integration/openapi/index.md <==
---
id: integration-openapi
---
# API Guidelines

This document defines the API design standards and implementation requirements for the Bluefly AI Platform, ensuring strict OpenAPI 3.1 compliance and resource-driven execution patterns.

## Core Principles

1. **OpenAPI 3.1 Compliance**: All APIs must be explicitly defined using OpenAPI 3.1
2. **Resource-Driven Design**: All operations follow resource-oriented patterns
3. **Full Platform Validation**: Continuous validation across all environments
4. **Enterprise Security**: JWT/OAuth2, RBAC, and automated scanning
5. **Observability**: Langfuse, Helicone, and OpenLLMetry integration

## OpenAPI Specification Requirements

All APIs must be defined using OpenAPI 3.1 and validated against our standards:

```bash
# Validate OpenAPI specification
bfcli openapi validate
bfcli openapi lint
bfcli openapi test
```

Required OpenAPI components:

```yaml
openapi: 3.1.0
info:
  title: Bluefly AI Platform API
  version: '1.0.0'
  description: |
    Enterprise AI Platform API
    - OpenAPI 3.1 compliant
    - Resource-driven execution
    - Full platform validation
  contact:
    name: Platform Team
    email: platform@bluefly.io
  license:
    name: Proprietary
servers:
  - url: https://api.bluefly.io/v1
    description: Production API
  - url: https://staging-api.bluefly.io/v1
    description: Staging API
components:
  securitySchemes:
    bearerAuth:
      type: http
      scheme: bearer
      bearerFormat: JWT
    apiKeyAuth:
      type: apiKey
      in: header
      name: X-API-Key
  schemas:
    Error:
      type: object
      required:
        - code
        - message
      properties:
        code:
          type: string
          enum: [invalid_request, unauthorized, forbidden, not_found, conflict, rate_limited, internal_error]
        message:
          type: string
        details:
          type: array
          items:
            $ref: '#/components/schemas/ErrorDetail'
        requestId:
          type: string
          format: uuid
    ErrorDetail:
      type: object
      required:
        - field
        - issue
      properties:
        field:
          type: string
        issue:
          type: string
        message:
          type: string
  responses:
    Error:
      description: Error response
      content:
        application/json:
          schema:
            $ref: '#/components/schemas/Error'
    Unauthorized:
      description: Authentication required
      content:
        application/json:
          schema:
            $ref: '#/components/schemas/Error'
    Forbidden:
      description: Insufficient permissions
      content:
        application/json:
          schema:
            $ref: '#/components/schemas/Error'
    NotFound:
      description: Resource not found
      content:
        application/json:
          schema:
            $ref: '#/components/schemas/Error'
    RateLimited:
      description: Rate limit exceeded
      content:
        application/json:
          schema:
            $ref: '#/components/schemas/Error'
      headers:
        Retry-After:
          schema:
            type: integer
            format: int32
```

## Resource-Driven Execution

All API endpoints must follow resource-oriented patterns:

1. **Resource Identification**
   - Use nouns for resource names
   - Follow hierarchical relationships
   - Maintain consistent naming conventions

2. **Resource Operations**
   - GET: Retrieve resource(s)
   - POST: Create resource
   - PUT: Replace resource
   - PATCH: Update resource
   - DELETE: Remove resource

3. **Resource Relationships**
   - Use nested resources for relationships
   - Maintain referential integrity
   - Follow RESTful principles

Example resource structure:

```yaml
paths:
  /models:
    get:
      summary: List models
      operationId: listModels
      tags: [Models]
      security:
        - bearerAuth: []
      parameters:
        - $ref: '#/components/parameters/PageSize'
        - $ref: '#/components/parameters/PageToken'
        - $ref: '#/components/parameters/Filter'
        - $ref: '#/components/parameters/Sort'
      responses:
        '200':
          description: Successful response
          content:
            application/json:
              schema:
                $ref: '#/components/schemas/ModelList'
        '401':
          $ref: '#/components/responses/Unauthorized'
        '403':
          $ref: '#/components/responses/Forbidden'
        '429':
          $ref: '#/components/responses/RateLimited'
  /models/{modelId}:
    parameters:
      - name: modelId
        in: path
        required: true
        schema:
          type: string
          format: uuid
    get:
      summary: Get model
      operationId: getModel
      tags: [Models]
      security:
        - bearerAuth: []
      responses:
        '200':
          description: Successful response
          content:
            application/json:
              schema:
                $ref: '#/components/schemas/Model'
        '401':
          $ref: '#/components/responses/Unauthorized'
        '403':
          $ref: '#/components/responses/Forbidden'
        '404':
          $ref: '#/components/responses/NotFound'
        '429':
          $ref: '#/components/responses/RateLimited'
```

## Response Standards

### Success Response

```json
{
  "data": {
    "id": "model-123",
    "type": "text-generation",
    "name": "GPT-4",
    "parameters": {
      "temperature": 0.7,
      "maxTokens": 1024
    },
    "metadata": {
      "team": "research",
      "project": "content-generation"
    },
    "createdAt": "2024-03-15T12:00:00Z",
    "updatedAt": "2024-03-15T14:30:00Z"
  },
  "meta": {
    "requestId": "req-abc-123",
    "timestamp": "2024-03-15T14:35:12Z",
    "version": "1.0.0"
  }
}
```

### Collection Response

```json
{
  "data": [
    {
      "id": "model-123",
      "type": "text-generation",
      "name": "GPT-4"
    }
  ],
  "meta": {
    "pagination": {
      "total": 48,
      "count": 1,
      "page": 1,
      "pages": 48,
      "next": "/api/v1/models?page=2&pageSize=1"
    },
    "requestId": "req-abc-123",
    "timestamp": "2024-03-15T14:35:12Z",
    "version": "1.0.0"
  }
}
```

## Error Handling

All errors must follow the standard error format:

```json
{
  "error": {
    "code": "invalid_request",
    "message": "The request was invalid",
    "details": [
      {
        "field": "name",
        "issue": "required",
        "message": "Name is required"
      }
    ],
    "requestId": "req-abc-123"
  }
}
```

## Security Requirements

1. **Authentication**
   - JWT Bearer tokens for all requests
   - API keys for service-to-service communication
   - OAuth2 for user authentication

2. **Authorization**
   - Role-based access control (RBAC)
   - Resource-level permissions
   - Audit logging for all operations

3. **Rate Limiting**
   - Per-endpoint rate limits
   - Per-client rate limits
   - Burst allowance for authenticated clients

## Observability

All endpoints must include:

1. **Request Tracking**
   - Unique request ID
   - Client ID
   - User ID (if authenticated)
   - Timestamp

2. **Performance Metrics**
   - Response time
   - Resource usage
   - Error rates

3. **Logging**
   - Request/response logging
   - Error logging
   - Audit logging

## Versioning

1. **API Versioning**
   - URL path versioning (/v1, /v2)
   - Content negotiation for minor versions
   - Deprecation notices for major versions

2. **Schema Versioning**
   - OpenAPI version tracking
   - Schema evolution guidelines
   - Backward compatibility requirements

## Platform Validation

Every API must pass:

1. **OpenAPI Validation**
   - Schema validation
   - Security validation
   - Documentation completeness

2. **Performance Testing**
   - Response time benchmarks
   - Throughput testing
   - Load testing

3. **Security Scanning**
   - Vulnerability scanning
   - Dependency scanning
   - Compliance checking

Use `bfcli api validate` to run full API validation.

## URL Structure

Follow these URL structure conventions:

```
https://api.example.com/api/v1/resource
https://api.example.com/api/v1/resource/{id}
https://api.example.com/api/v1/resource/{id}/subresource
```

- Base path includes `/api/v{version_number}/`
- Use plural nouns for resources (e.g., `/models` not `/model`)
- Use kebab-case for multi-word resource names (e.g., `/model-variants`)
- Use query parameters for filtering, sorting, and pagination

## HTTP Methods

Use standard HTTP methods appropriately:

| Method | Usage | Example |
|--------|-------|---------|
| GET | Retrieve resources | GET /api/v1/models |
| POST | Create resources | POST /api/v1/models |
| PUT | Replace resources | PUT /api/v1/models/123 |
| PATCH | Partial update | PATCH /api/v1/models/123 |
| DELETE | Remove resources | DELETE /api/v1/models/123 |

## Request & Response Format

### Request Structure

```json
{
  "name": "My Custom Model",
  "type": "text-generation",
  "parameters": {
    "temperature": 0.7,
    "max_tokens": 1024
  },
  "metadata": {
    "team": "research",
    "project": "content-generation"
  }
}
```

- Use camelCase for property names
- Group related properties into nested objects
- Keep the structure flat when possible
- Use appropriate data types

### Response Structure

```json
{
  "data": {
    "id": "model-123",
    "name": "My Custom Model",
    "type": "text-generation",
    "parameters": {
      "temperature": 0.7,
      "max_tokens": 1024
    },
    "metadata": {
      "team": "research",
      "project": "content-generation"
    },
    "createdAt": "2025-05-01T12:00:00Z",
    "updatedAt": "2025-05-15T14:30:00Z"
  },
  "meta": {
    "requestId": "req-abc-123",
    "timestamp": "2025-05-15T14:35:12Z"
  }
}
```

For collections:

```json
{
  "data": [
    {
      "id": "model-123",
      "name": "GPT-4",
      "type": "text-generation"
    },
    {
      "id": "model-456",
      "name": "DALL-E 3",
      "type": "image-generation"
    }
  ],
  "meta": {
    "pagination": {
      "total": 48,
      "count": 2,
      "page": 1,
      "pages": 24,
      "prev": null,
      "next": "/api/v1/models?page=2"
    },
    "requestId": "req-abc-123",
    "timestamp": "2025-05-15T14:35:12Z"
  }
}
```

## Error Handling

Use consistent error response format:

```json
{
  "error": {
    "code": "invalid_request",
    "message": "The request was invalid",
    "details": [
      {
        "field": "name",
        "issue": "required",
        "message": "Name is required"
      }
    ],
    "requestId": "req-abc-123"
  }
}
```

HTTP status codes:

| Code | Meaning | Example Usage |
|------|---------|---------------|
| 200 | OK | Successful GET, PUT, PATCH |
| 201 | Created | Successful POST that creates a resource |
| 204 | No Content | Successful DELETE |
| 400 | Bad Request | Invalid request format or parameters |
| 401 | Unauthorized | Missing or invalid authentication |
| 403 | Forbidden | Valid authentication but insufficient permissions |
| 404 | Not Found | Resource doesn't exist |
| 409 | Conflict | Resource already exists or other conflict |
| 422 | Unprocessable Entity | Semantic errors in request |
| 429 | Too Many Requests | Rate limit exceeded |
| 500 | Internal Server Error | Server-side error |

## Authentication

Use JWT Bearer tokens for authentication:

```
Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...
```

Include token validation middleware in all protected routes:

```javascript
app.use('/api/v1', authenticate(), routes);
```

## Versioning

Include version in the URL path:

```
/api/v1/models
/api/v2/models
```

Version compatibility guidelines:

- Minor version changes must be backward compatible
- Major version changes may break compatibility
- Maintain previous major versions for at least 6 months
- Document all breaking changes in the changelog

## Rate Limiting

Implement rate limiting on all endpoints:

```
X-RateLimit-Limit: 100
X-RateLimit-Remaining: 95
X-RateLimit-Reset: 1622568000
```

When limit is exceeded, return 429 Too Many Requests with Retry-After header.

## Pagination

Use cursor-based pagination for large collections:

```
/api/v1/models?limit=10&after=model-123
```

Include pagination metadata in response:

```json
"meta": {
  "pagination": {
    "total": 48,
    "count": 10,
    "after": "model-123",
    "hasMore": true,
    "next": "/api/v1/models?limit=10&after=model-132"
  }
}
```

## Filtering & Sorting

Support filtering via query parameters:

```
/api/v1/models?type=text-generation&status=active
```

Support sorting via `sort` parameter:

```
/api/v1/models?sort=name       # Ascending
/api/v1/models?sort=-name      # Descending
/api/v1/models?sort=type,-name # Multiple fields
```

## CORS Configuration

Configure CORS to allow necessary domains:

```javascript
app.use(cors({
  origin: ['https://app.example.com', 'https://admin.example.com'],
  methods: ['GET', 'POST', 'PUT', 'PATCH', 'DELETE'],
  allowedHeaders: ['Content-Type', 'Authorization'],
  exposedHeaders: ['X-RateLimit-Limit', 'X-RateLimit-Remaining', 'X-RateLimit-Reset'],
  credentials: true,
  maxAge: 86400 // 24 hours
}));
```

## Caching

Use HTTP caching headers:

```
Cache-Control: public, max-age=60
ETag: "33a64df551425fcc55e4d42a148795d9f25f89d4"
Last-Modified: Wed, 15 May 2025 14:30:00 GMT
```

Implement conditional requests with If-None-Match and If-Modified-Since headers.

## API Documentation

Document all APIs using OpenAPI specification and provide interactive documentation:

```bash
# Generate API documentation
npm run generate-api-docs

# Serve API documentation
npm run serve-api-docs
```

Documentation should include:

- Endpoint descriptions
- Request/response schemas
- Authentication requirements
- Error codes and messages
- Example requests and responses
- Rate limiting information

## API Validation & Testing

All APIs must have:

1. Schema validation for requests and responses
2. Automated tests for all endpoints
3. Contract tests to ensure API compatibility
4. Performance tests to ensure response time requirements

Example validation:

```javascript
const schema = Joi.object({
  name: Joi.string().required(),
  type: Joi.string().valid('text-generation', 'image-generation').required(),
  parameters: Joi.object({
    temperature: Joi.number().min(0).max(2).default(1),
    max_tokens: Joi.number().integer().min(1).max(4096).default(1024)
  }).default({})
});

const { error, value } = schema.validate(req.body);
if (error) {
  return res.status(400).json({
    error: {
      code: 'invalid_request',
      message: 'The request was invalid',
      details: error.details.map(d => ({
        field: d.path.join('.'),
        issue: d.type,
        message: d.message
      })),
      requestId: req.id
    }
  });
}
```

## API Gateway Configuration

All APIs are exposed through the API Gateway (bfapi) which provides:

- Authentication and authorization
- Rate limiting and throttling
- Request validation
- Response caching
- Logging and monitoring
- API key management

## API Monitoring

All APIs are monitored for:

- Request volume
- Response time
- Error rates
- Authentication failures
- Rate limit hits

Metrics are exposed via Prometheus endpoints and visualized in Grafana dashboards.

## Changelog

Document all API changes in a changelog:

```markdown
# API Changelog

## v1.1.0 - 2025-05-15

### Added
- New `status` field to model responses
- Support for sorting models by creation date

### Changed
- Increased rate limits from 100 to 200 requests per minute

## v1.0.0 - 2025-04-01

- Initial release of the API
```
